/*
    Copyright 2012  Dan Vratil <dan@progdan.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERVICESCONTACTSTESTS_H
#define SERVICESCONTACTSTESTS_H

#include <qtest_kde.h>

namespace KGAPI {
namespace Objects {
class Contact;
class ContactsGroup;
}
}

class ServicesContactsTests : public QObject
{
    Q_OBJECT

  private:
    void compareContacts(const KGAPI::Objects::Contact *contact, const QSettings &values, bool fromSerializer = false);
    void compareGroups(const KGAPI::Objects::ContactsGroup *group, const QSettings &values, bool fromSerializer = false);

  private Q_SLOTS:
    void testContactsParsers();
    void testContactsGroupParsers();

    void testContactsSerializer();
    void testContactsGroupSerializer();

    void testUrls();
};

#endif // SERVICESCONTACTSTESTS_H
