/*
 * This file is part of LibKGAPI library
 *
 * Copyright (C) 2013  Daniel Vrátil <dvratil@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef LIBKGAPI2_DRIVEPARENTREFERENCE_P_H
#define LIBKGAPI2_DRIVEPARENTREFERENCE_P_H

#include "parentreference.h"

#include <QtCore/QVariantMap>

namespace KGAPI2
{

namespace Drive
{

class ParentReference::Private
{
  public:
    Private();
    Private(const Private &other);

    QString id;
    QUrl selfLink;
    QUrl parentLink;
    bool isRoot;

    static ParentReferencePtr fromJSON(const QVariantMap &map);
    static QVariantMap toJSON(const ParentReferencePtr &reference);
};

} // namespace Drive

} // namespace KGAPI2

#endif // LIBKGAPI2_DRIVEPARENTREFERENCE_P_H
